@@ -1,5 +1,5 @@
 /*
- * (C) 2001 	Fatih Demir <kabalak@gtranslator.org>
+ * (C) 2001-2003 	Fatih Demir <kabalak@gtranslator.org>
  *
  * gtranslator is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
  */
 
 #include "bookmark.h"
+#include "dialogs.h"
+#include "gui.h"
 #include "message.h"
 #include "nautilus-string.h"
 #include "open.h"
@@ -25,6 +27,8 @@
 #include "preferences.h"
 #include "utils.h"
 
+#include <libgnomeui/gnome-app-helper.h>
+
 /*
  * For the moment 10 bookmarks should be the upper limit.
  */
@@ -36,6 +40,14 @@
  */
 GList *gtranslator_bookmarks=NULL;
 
+/*
+ * Shamelessly stolen from our history.c for the moment, it's a duplication of work
+ *  maybe we should concentrate this in a nearby time but for now it shall work.
+ */
+void gtranslator_open_file_dialog_from_bookmark(GtkWidget *widget, gchar *filename);
+void free_userdata_bookmark(GtkWidget *widget, gpointer userdata);
+gchar *gtranslator_bookmark_escape(const gchar *str);
+
 /*
  * Create and return a GtrBookmark from the current position & po file -- 
  *  if a file is opened yet.
@@ -52,6 +64,8 @@ GtrBookmark *gtranslator_bookmark_new()
 	
 	bookmark->position=g_list_position(po->messages, po->current);
 
+	bookmark->comment=g_strdup("No comment");
+
 	if(bookmark->position < 0)
 	{
 		bookmark->position=0;
@@ -60,15 +74,36 @@ GtrBookmark *gtranslator_bookmark_new()
 	return bookmark;
 }
 
+/*
+ * Return the GtrBookmark with the given comment, yeah!
+ */
+GtrBookmark *gtranslator_bookmark_new_with_comment(const gchar *comment)
+{
+	GtrBookmark *nbookmark;
+
+	nbookmark=gtranslator_bookmark_new();
+	g_return_val_if_fail(nbookmark!=NULL, NULL);
+
+	if(comment)
+	{
+		gtranslator_bookmark_set_comment(nbookmark, comment);
+	}
+	else
+	{
+		gtranslator_bookmark_set_comment(nbookmark, _("No comment"));
+	}
+
+	return nbookmark;
+}
+
 /*
  * Creates the bookmark from a given gchar-string formatted by gtranslator.
  */
 GtrBookmark *gtranslator_bookmark_new_from_string(const gchar *string)
 {
 	GtrBookmark 	*bookmark=g_new0(GtrBookmark, 1);
-	gchar 	**values;
-	gchar 	*filename,
-		*encoding_area;
+	gchar 		**values;
+	gchar 		*filename, *encoding_area;
 
 	g_return_val_if_fail(string!=NULL, NULL);
 
@@ -88,28 +123,34 @@ GtrBookmark *gtranslator_bookmark_new_from_string(const gchar *string)
 		gchar *tempzulu;
 		
 		tempzulu=nautilus_str_get_prefix(string, "#");
-		g_strreverse(tempzulu);
-
-		/*
-		 * Some reverse-thinking...
-		 */
-		filename=nautilus_str_get_prefix(tempzulu,
-			":kramkoob_rotalsnartg");
+		filename=nautilus_str_get_after_prefix(tempzulu, ":");
+		filename++;
 		
 		g_free(tempzulu);
 		g_return_val_if_fail(filename!=NULL, NULL);
 	}
 	
 	bookmark->file=g_strdup(filename);
-	g_free(filename);
+	/*
+	 * FIXME: Doesn't work due to the filename++ above, fix needed!
+	 * 
+	 * g_free(filename);
+	 */
 	
 	/*
 	 * Operate on the resting parts of the string-encoded bookmark and
 	 *  split it up into its normally 4 parts.
 	 */
 	encoding_area=nautilus_str_get_after_prefix(string, "#");
-	values=g_strsplit(encoding_area, "/", 2);
-	g_free(encoding_area);
+	encoding_area++;
+
+	values=g_strsplit(encoding_area, "/", 3);
+
+	/*
+	 * FIXME: Again the str++ problem like above with filename++, aaaarg!
+	 * 
+	 * g_free(encoding_area);
+	 */
 
 	bookmark->version = g_strdup(values[0]);
 	
@@ -126,6 +167,15 @@ GtrBookmark *gtranslator_bookmark_new_from_string(const gchar *string)
 		bookmark->position=-1;
 	}
 
+	if(values[2])
+	{
+		bookmark->comment=g_strdup(values[2]);
+	}
+	else
+	{
+		bookmark->comment=g_strdup("No comment");
+	}
+
 	g_strfreev(values);
 	return bookmark;
 }
@@ -139,8 +189,8 @@ gchar *gtranslator_bookmark_string_from_bookmark(GtrBookmark *bookmark)
 
 	g_return_val_if_fail(bookmark!=NULL, NULL);
 
-	string=g_strdup_printf("gtranslator_bookmark:%s#%s/%i",
-		bookmark->file,  bookmark->version, bookmark->position);
+	string=g_strdup_printf("gtranslator_bookmark:%s#%s/%i/%s",
+		bookmark->file, bookmark->version, bookmark->position, bookmark->comment);
 
 	return string;
 }
@@ -170,6 +220,7 @@ gchar *gtranslator_bookmark_new_bookmark_string()
 void gtranslator_bookmark_open(GtrBookmark *bookmark)
 {
 	g_return_if_fail(bookmark!=NULL);
+	g_return_if_fail(bookmark->file!=NULL);
 
 	/*
 	 * Open the po file.
@@ -246,7 +297,7 @@ gboolean gtranslator_bookmark_resolvable(GtrBookmark *bookmark)
 			 * Free the "new-old" po and reassign the saved
 			 *  original po variable.
 			 */
-			gtranslator_po_free();
+			gtranslator_po_free(po);
 			po=current_po;
 
 			return TRUE;
@@ -340,6 +391,34 @@ void gtranslator_bookmark_add(GtrBookmark *bookmark)
 	}
 }
 
+/*
+ * Adding the bookmark directly without any feedback and such but easyness.
+ */
+void gtranslator_bookmark_add_direct()
+{
+	GtrBookmark *bookmark=NULL;
+
+	bookmark=gtranslator_bookmark_new();
+	g_return_if_fail(bookmark!=NULL);
+
+	gtranslator_bookmark_add(GTR_BOOKMARK(bookmark));
+	gtranslator_bookmark_free(bookmark);
+}
+
+void gtranslator_bookmark_add_direct_with_comment(const gchar *comment)
+{
+	GtrBookmark *bookmark=NULL;
+
+	bookmark=gtranslator_bookmark_new();
+	g_return_if_fail(bookmark!=NULL);
+	g_return_if_fail(comment!=NULL);
+
+	gtranslator_bookmark_set_comment(GTR_BOOKMARK(bookmark), comment);
+
+	gtranslator_bookmark_add(GTR_BOOKMARK(bookmark));
+	gtranslator_bookmark_free(bookmark);
+}
+
 /*
  * Remove the given GtrBookmark from our list -- the elements should 
  *  be matching "enough" to apply the removal from the list.
@@ -505,6 +584,103 @@ void gtranslator_bookmark_save_list()
 	}
 }
 
+/*
+ * Stolen from history.c -> working for the moment but should be reorganised later on.
+ */
+void gtranslator_bookmark_show_list(void)
+{
+	static gint len = 0;
+	gint i;
+	GnomeUIInfo *menu;
+	GList *list, *onelist;
+	GtrBookmark *bookmark;
+	
+	gchar *menupath = _("_View/_Bookmarks/");
+
+	/*
+	 * Delete the old entries.
+	 */
+	gnome_app_remove_menu_range(GNOME_APP(gtranslator_application), menupath, 0, len);
+
+	/*
+	 * Get the old history entries.
+	 */ 
+	list=gtranslator_bookmark_get_list();
+
+	i=len=g_list_length(list);
+
+	/*
+	 * Parse the list.
+	 */
+	for(onelist=g_list_last(list); onelist!=NULL; onelist=g_list_previous(onelist))
+	{
+		/*
+		 * Get the history entry.
+		 */
+		bookmark=GTR_BOOKMARK(onelist->data);
+
+		menu=g_new0(GnomeUIInfo, 2);
+
+		/*
+		 * Set the label name.
+		 */
+		menu->label=g_strdup_printf("_%i: %s (%s, %i)", i--,
+			gtranslator_bookmark_escape(bookmark->file),
+			bookmark->version, bookmark->position);
+		
+		/*
+		 * Set the GnomeUIInfo settings and labels.
+		 */
+		menu->type=GNOME_APP_UI_ITEM;
+		menu->hint=g_strdup_printf(_("Open %s (%s)"), bookmark->file, bookmark->comment);
+		menu->moreinfo=(gpointer)gtranslator_open_file_dialog_from_bookmark;
+		menu->user_data=bookmark->file;
+		(menu+1)->type=GNOME_APP_UI_ENDOFINFO;
+
+		/*
+		 * Insert this item into menu
+		 */
+		gnome_app_insert_menus(GNOME_APP(gtranslator_application), menupath, menu);
+		gnome_app_install_menu_hints(GNOME_APP(gtranslator_application), menu);
+
+		g_signal_connect(GTK_OBJECT(menu->widget), "destroy",
+				   GTK_SIGNAL_FUNC(free_userdata_bookmark), (gpointer) menu->hint);
+
+		/*
+		 * Free the string and the GnomeUIInfo structure.
+		 */
+		GTR_FREE((gpointer) menu->label);
+		GTR_FREE(menu);
+	}
+}
+
+void free_userdata_bookmark(GtkWidget *widget, gpointer userdata)
+{
+	GTR_FREE(userdata);
+}
+
+void gtranslator_open_file_dialog_from_bookmark(GtkWidget *widget, gchar *filename)
+{
+	if (!gtranslator_should_the_file_be_saved_dialog())
+		return;
+	gtranslator_file_close(NULL, NULL);
+
+	gtranslator_open_file(filename);
+}
+
+/*
+ * Escape the menu display items rightly.
+ */
+gchar *gtranslator_bookmark_escape(const gchar *str)
+{
+	gchar	*display_str=NULL;
+	
+	g_return_val_if_fail(str!=NULL, NULL);
+
+	display_str=nautilus_str_replace_substring(str, "_", "__");
+	return display_str;
+}
+
 /*
  * Return a copy of our internally used bookmarks' list -- or NULL.
  */
@@ -520,6 +696,22 @@ GList *gtranslator_bookmark_get_list()
 	}
 }
 
+/*
+ * Change the comment of the given GtrBookmark with the given gchar.
+ */
+void gtranslator_bookmark_set_comment(GtrBookmark *bookmark, const gchar *newcomment)
+{
+	g_return_if_fail(bookmark!=NULL);
+	g_return_if_fail(newcomment!=NULL);
+
+	if(GTR_BOOKMARK(bookmark)->comment && GTR_BOOKMARK(bookmark)->comment[0]!='\0')
+	{
+		GTR_FREE(GTR_BOOKMARK(bookmark)->comment);
+	}
+
+	GTR_BOOKMARK(bookmark)->comment=g_strdup(newcomment);
+}
+
 /*
  * A convenience copy function for our GtrBookmarks.
  */
@@ -530,7 +722,8 @@ GtrBookmark *gtranslator_bookmark_copy(GtrBookmark *bookmark)
 	g_return_val_if_fail(bookmark!=NULL, NULL);
 
 	copy->file=g_strdup(bookmark->file);
-	copy->version = g_strdup(bookmark->version);
+	copy->version=g_strdup(bookmark->version);
+	copy->comment=g_strdup(bookmark->comment);
 	copy->position=bookmark->position;
 
 	return copy;
@@ -545,6 +738,7 @@ void gtranslator_bookmark_free(GtrBookmark *bookmark)
 	{
 		g_free(bookmark->file);
 		g_free(bookmark->version);
+		g_free(bookmark->comment);
 		g_free(bookmark);
 	}
 }
