@@ -3,6 +3,7 @@
  *			Gediminas Paulauskas <menesis@gtranslator.org>
  *			Peeter Vois <peeter@gtranslator.org>
  *			Thomas Ziehmer <thomas@gtranslator.org>
+ *			Ross Golder <ross@gtranslator.org>
  *
  * gtranslator is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,11 +27,14 @@
 
 #include "about.h"
 #include "actions.h"
+#include "bookmark.h"
 #include "color-schemes.h"
 #include "dialogs.h"
 #include "dnd.h"
 #include "find.h"
-#include "gtkspell.h"
+#ifdef NOT_PORTED
+# include "gtkspell.h"
+#endif
 #include "gui.h"
 #include "header_stuff.h"
 #include "history.h"
@@ -76,6 +80,12 @@ GtkWidget *gtranslator_application_bar;
 
 GtkWidget *content_pane;
 GtkWidget *table_pane;
+#ifdef NOT_PORTED
+guint text_box_insert_text_signal_id;
+guint text_box_delete_text_signal_id;
+#endif
+guint trans_box_insert_text_signal_id;
+guint trans_box_delete_text_signal_id;
 
 /*
  * The comment/extra content area in the main window.
@@ -98,11 +108,6 @@ static void selection_get_handler(GtkWidget *widget,
 	GtkSelectionData *selection_data, guint info,
 	guint time_stamp, gpointer data);
 
-/*
- * To get the left/right moves from the cursor.
- */ 
-static gint gtranslator_keyhandler(GtkWidget *widget, GdkEventKey *event);
-
 /*
  * The target formats
  */
@@ -294,30 +299,30 @@ void gtranslator_create_main_window(void)
 			 G_CALLBACK(selection_get_handler), NULL);
 	g_signal_connect_after(G_OBJECT(trans_box), "selection-get",
 			 G_CALLBACK(selection_get_handler), NULL);
-/*
-	g_signal_connect(G_OBJECT(gtk_text_view_get_buffer(text_box)), 
-			 "insert-text",
-			 G_CALLBACK(insert_text_handler), NULL);
-	g_signal_connect(G_OBJECT(gtk_text_view_get_buffer(text_box)),
-			 "delete-range",
-			 G_CALLBACK(delete_text_handler), NULL);
-*/
 
-/*	g_signal_connect(G_OBJECT(gtk_text_view_get_buffer(trans_box)), 
-			 "insert-text",
-			 G_CALLBACK(insert_text_handler), NULL);
-
-	g_signal_connect(G_OBJECT(gtk_text_view_get_buffer(trans_box)), 
-			 "delete-range",
-			 G_CALLBACK(delete_text_handler), NULL);
-*/
+#ifdef NOT_PORTED
+	text_box_insert_text_signal_id = g_signal_connect(
+		G_OBJECT(gtk_text_view_get_buffer(text_box)), 
+		"insert-text",
+		G_CALLBACK(insert_text_handler), NULL);
+	text_box_delete_text_signal_id = g_signal_connect(
+		G_OBJECT(gtk_text_view_get_buffer(text_box)),
+		"delete-range",
+		G_CALLBACK(delete_text_handler), NULL);
+#endif
+	trans_box_insert_text_signal_id = g_signal_connect(
+		G_OBJECT(gtk_text_view_get_buffer(trans_box)), 
+		"insert-text",
+		G_CALLBACK(insert_text_handler), NULL);
+	trans_box_delete_text_signal_id = g_signal_connect(
+		G_OBJECT(gtk_text_view_get_buffer(trans_box)), 
+		"delete-range",
+		G_CALLBACK(delete_text_handler), NULL);
 	
 	g_signal_connect(G_OBJECT(gtk_text_view_get_buffer(trans_box)), 
 			 "changed",
 			 G_CALLBACK(gtranslator_translation_changed), NULL);
 	
-	g_signal_connect(G_OBJECT(gtranslator_application), "key-press-event",
-			 G_CALLBACK(gtranslator_keyhandler), NULL);
 	/*
 	 * The D'n'D signals
 	 */
@@ -351,24 +356,24 @@ void delete_text_handler(GtkTextBuffer *textbuf, GtkTextIter *start,
 		fake_string=gtk_text_buffer_get_text(textbuf, 
 						     start, end, FALSE);
 		
-//		gtranslator_undo_register_deletion(fake_string, start_position);
+//		gtranslator_undo_register_deletion(fake_string, start, end);
 		GTR_FREE(fake_string);
 	}
 	gtranslator_delete_highlighted(textbuf, start, end, NULL);
-	g_signal_stop_emission_by_name (G_OBJECT (textbuf), "delete_text");
+	g_signal_stop_emission_by_name (G_OBJECT (textbuf), "delete-text");
 }
 
 /*
  * The own quit-code
  */
-gint gtranslator_quit(GtkWidget  * widget, GdkEventAny  * e,
+void gtranslator_quit(GtkWidget  * widget, GdkEventAny  * e,
 			     gpointer useless)
 {
 	/*
 	 * If file was changed, but user pressed Cancel, don't quit
 	 */
 	if (!gtranslator_should_the_file_be_saved_dialog())
-		return TRUE;
+		return;
 
 	gtranslator_file_close(NULL, NULL);
 
@@ -388,14 +393,6 @@ gint gtranslator_quit(GtkWidget  * widget, GdkEventAny  * e,
 	 */
 	gtranslator_config_set_string("runtime/filename", "--- No file ---");
 
-	/*
-	 * Save the messages table state.
-	 */
-	if(GtrPreferences.show_messages_table)
-	{
-		gtranslator_messages_table_save_state();
-	}
-
 	/*
 	 * Shutdown our internal learning system.
 	 */
@@ -404,6 +401,11 @@ gint gtranslator_quit(GtkWidget  * widget, GdkEventAny  * e,
 		gtranslator_learn_shutdown();
 	}
 
+	/*
+	 * Now save the bookmarks' list before closing/exiting the application.
+	 */
+	gtranslator_bookmark_save_list();
+
 	/*
 	 * Free the used GtrTranslator structure.
 	 */
@@ -457,7 +459,7 @@ gint gtranslator_quit(GtkWidget  * widget, GdkEventAny  * e,
 	 * Quit with the normal Gtk+ quit.
 	 */
 	gtk_main_quit();
-	return FALSE;
+	return;
 }
 
 /*
@@ -532,9 +534,20 @@ void gtranslator_application_bar_update(gint pos)
 	}
 	
 	/*
-	 * Assign the first part.
+	 * Assign the toolbar message string according to the message quality.
+	 *
+	 * If it's a plural forms message we do have got a more enhanced and
+	 *  longer message there indicating a plural forms message.
+	 *
 	 */
-	str=g_strdup_printf(_("Message %d / %d / Status: %s"), pos + 1, po->length, status);
+	if(msg->msgid_plural)
+	{
+		str=g_strdup_printf(_("Message %d / %d / Status: %s (Message contains plural forms)"), pos + 1, po->length, status);
+	}
+	else
+	{
+		str=g_strdup_printf(_("Message %d / %d / Status: %s"), pos + 1, po->length, status);
+	}
 	
 	/*
 	 * Set the appbar text.
@@ -607,6 +620,7 @@ void gtranslator_translation_changed(GtkWidget  *buffer, gpointer useless)
 		     && (msg->status & GTR_MSG_STATUS_FUZZY))
 		{
 		     	gtranslator_message_status_set_fuzzy(msg, FALSE);
+			po->fuzzy--;
 			gtk_check_menu_item_set_active(
 				GTK_CHECK_MENU_ITEM(the_edit_menu[19].widget),
 				FALSE
@@ -614,7 +628,7 @@ void gtranslator_translation_changed(GtkWidget  *buffer, gpointer useless)
 		}
 		if(GtrPreferences.show_messages_table)
 		{
-			gtranslator_messages_table_update_message_status(msg);
+			gtranslator_messages_table_update_row(msg);
 		}
 	}
 }
@@ -634,7 +648,7 @@ void insert_text_handler (GtkTextBuffer *textbuffer, GtkTextIter *pos,
 		/*
 		* Register the text for an insertion undo action.
 		*/
-//		gtranslator_undo_register_insertion(text, *position);
+//		gtranslator_undo_register_insertion(text, pos);
 
 		if(GtrPreferences.dot_char)
 		{
@@ -642,12 +656,21 @@ void insert_text_handler (GtkTextBuffer *textbuffer, GtkTextIter *pos,
 			old = result;
 			result = gtranslator_utils_invert_dot((gchar*)result);
 			g_free(old);
+			length = strlen(result);
 		}
 	}
-	gtranslator_insert_highlighted(textbuffer, pos, result, length, NULL);
+
+#ifdef NOT_PORTED
+	gtranslator_insert_highlighted(textbuffer, pos, result, length, data);
+#endif
+	g_signal_handler_block(textbuffer, trans_box_insert_text_signal_id);
+	gtk_text_buffer_insert(textbuffer, pos, result, length);
+	g_signal_handler_unblock(textbuffer, trans_box_insert_text_signal_id);
+
+	g_signal_stop_emission_by_name (G_OBJECT(textbuffer), "insert-text");
+	//gtk_text_buffer_place_cursor(textbuffer, pos);
+
 	g_free(result);
-	g_signal_stop_emission_by_name (G_OBJECT(textbuffer), "insert_text");
-	gtk_text_buffer_place_cursor(textbuffer, pos);
 }
 
 void selection_get_handler(GtkWidget *widget, GtkSelectionData *selection_data,
@@ -666,71 +689,3 @@ void selection_get_handler(GtkWidget *widget, GtkSelectionData *selection_data,
 	g_free(text);
 	g_free(undotted_text);
 }
-
-/*
- * The own keyhandler to get the left/right/up/down actions.
- */ 
-static gint gtranslator_keyhandler(GtkWidget *widget, GdkEventKey *event)
-{
-	g_return_val_if_fail(widget!=NULL, FALSE);
-	g_return_val_if_fail(event!=NULL, FALSE);
-	#define IfGood(x) \
-	if(GTK_WIDGET_SENSITIVE(GTK_WIDGET(((GnomeUIInfo)(x)).widget)))
-
-	if(file_opened)
-	{
-		/*
-		 * If we're having a Ctrl+Shift+Arrow Key issue, then respect
-		 *  this to be something another then just Ctrl+Arrow Key.
-		 */
-		if(event->state & GDK_SHIFT_MASK)
-		{
-			return FALSE;
-		}
-
-		/*
-		 * Now handle out "normal" navigation keys consisting out of
-		 *  Ctrl+Arrow Key.
-		 */
-		if(event->state & GDK_CONTROL_MASK)
-		{
-			switch(event->keyval)
-			{
-				case GDK_Left:
-				case GDK_Up:
-					IfGood(the_navibar[1])
-					{
-						gtranslator_message_go_to_previous(NULL, NULL);
-					}
-					break;
-				
-				case GDK_Right:
-				case GDK_Down:
-					IfGood(the_navibar[3])
-					{
-						gtranslator_message_go_to_next(NULL, NULL);
-					}
-					break;
-
-				case GDK_Page_Up:
-					IfGood(the_navibar[0])
-					{
-						gtranslator_message_go_to_first(NULL, NULL);
-					}
-					break;
-					
-				case GDK_Page_Down:
-					IfGood(the_navibar[4])
-					{
-						gtranslator_message_go_to_last(NULL, NULL);
-					}
-					break;
-					
-				default:
-					break;
-			}
-		}
-	}
-	
-	return FALSE;
-}
