@@ -226,14 +226,14 @@ void gtranslator_history_show(void)
 		{
 			menu->label=g_strdup_printf("_%i: %s -- %s", i--,
 				gtranslator_history_escape(entry->project_name),
-				gtranslator_history_escape(g_basename(entry->filename)));
+				gtranslator_history_escape(g_path_get_basename(entry->filename)));
 		}
 		else
 		{
 			menu->label=g_strdup_printf("_%i: %s %s -- %s", i--,
 				gtranslator_history_escape(entry->project_name),
 				gtranslator_history_escape(entry->project_version),
-				gtranslator_history_escape(g_basename(entry->filename)));
+				gtranslator_history_escape(g_path_get_basename(entry->filename)));
 		}
 		
 		/*
@@ -251,7 +251,7 @@ void gtranslator_history_show(void)
 		gnome_app_insert_menus(GNOME_APP(gtranslator_application), menupath, menu);
 		gnome_app_install_menu_hints(GNOME_APP(gtranslator_application), menu);
 
-		gtk_signal_connect(GTK_OBJECT(menu->widget), "destroy",
+		g_signal_connect(GTK_OBJECT(menu->widget), "destroy",
 				   GTK_SIGNAL_FUNC(free_userdata), (gpointer) menu->hint);
 
 		/*
