@@ -118,7 +118,7 @@ GtrLanguage languages[] = {
 	 "et@li.org", "8bit"
 	},
 	{
-	 N_("Farsi"), "fa", "isiri-3342",
+	 N_("Persian"), "fa", "isiri-3342",
 	 "", "8bit"
 	},
 	{
@@ -153,6 +153,10 @@ GtrLanguage languages[] = {
 	 N_("Hungarian"), "hu", "iso-8859-2",
 	 "hu@gnome.hu", "8bit"
 	},
+	{
+	 N_("Icelandic"), "is", "iso-8859-1",
+	 "gnome@techattack.nu", "8bit"
+	},
 	{
 	 N_("Indonesian"), "id", "iso-8859-1",
 	 "", "8bit"
