@@ -1,6 +1,7 @@
 /*
- * (C) 2000-2001 	Fatih Demir <kabalak@gtranslator.org>
+ * (C) 2000-2003 	Fatih Demir <kabalak@gtranslator.org>
  *			Gediminas Paulauskas <menesis@gtranslator.org>
+ *			Ross Golder <ross@gtranslator.org>
  *
  * gtranslator is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
 #define GTR_MESSAGES_H 1
 
 #include <glib.h>
+#include <gtk/gtktreemodel.h>
 #include "comment.h"
 
 /*
@@ -39,7 +41,14 @@ typedef enum {
 	 * Currently this is only internally useful but when using extended
 	 *  message comments this status might also get "persistent".
 	 */
-	GTR_MSG_STATUS_STICK		= 1 << 2
+	GTR_MSG_STATUS_STICK		= 1 << 2,
+
+	/*
+	 * "Review" could be another virtual/semi-conservative message status
+	 *   with which a translator could mark a message for review via this
+	 *    tag in the comment of the message for example.
+	 */
+	GTR_MSG_STATUS_REVIEW		= 1 << 3
 } GtrMsgStatus;
 
 /*
@@ -53,8 +62,22 @@ typedef struct {
 	
 	gint		no;
 	gint 		pos;
+
+	gchar		*msgid_plural;
+
+	/*
+	 * The plural forms msgstrs - the first "msgstr[0]" is stored in the
+	 *  1st message translation equivalent "msgstr" as it should be logical.
+	 */
+	gchar		*msgstr_1;
+	gchar		*msgstr_2;
 	
 	GtrMsgStatus 	status;
+
+	/*
+	 * GtkTreeIter for when using the messages table
+	 */
+	GtkTreeIter     iter;
 } GtrMsg;
 
 /*
