@@ -1,6 +1,7 @@
 /*
- * (C) 2000-2001 	Fatih Demir <kabalak@gtranslator.org>
+ * (C) 2000-2003 	Fatih Demir <kabalak@gtranslator.org>
  *			Gediminas Paulauskas <menesis@gtranslator.org>
+ *			Ross Golder <ross@gtranslator.org>
  *
  * gtranslator is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +33,9 @@
 #include "actions.h"
 #include "comment.h"
 #include "dialogs.h"
-#include "gtkspell.h"
+#ifdef NOT_PORTED
+# include "gtkspell.h"
+#endif
 #include "gui.h"
 #include "history.h"
 #include "message.h"
@@ -45,7 +48,9 @@
 #include "save.h"
 #include "translator.h"
 #include "undo.h"
-#include "utf8.h"
+#ifdef UTF8_CODE
+# include "utf8.h"
+#endif
 #include "utils.h"
 #include "utils_gui.h"
 
@@ -64,12 +69,13 @@ GtrPo *po;
 gboolean file_opened;
 gboolean message_changed;
 guint autosave_source_tag=1;
+gboolean open_anyway=FALSE;
 
 /*
  * These are to be used only inside this file
  */
 static void append_line(gchar ** old, const gchar * tail, gboolean continuation);
-static gboolean add_to_obsolete(gchar *comment);
+static gboolean add_to_obsolete(GtrPo *po, gchar *comment);
 static void write_the_message(gpointer data, gpointer fs);
 static gchar *restore_msg(const gchar * given);
 static void determine_translation_status(gpointer data, gpointer useless_stuff);
@@ -94,7 +100,7 @@ static void check_msg_status(GtrMsg * msg)
  */
 static void append_line(gchar ** old, const gchar * tail, gboolean continuation)
 {
-	gchar *to_add = g_new(gchar, strlen(tail));
+	gchar *to_add = g_new(gchar, strlen(tail) * 2);
 	gchar *result;
 	gint s, d = 0;
 
@@ -102,6 +108,13 @@ static void append_line(gchar ** old, const gchar * tail, gboolean continuation)
 		to_add[d++] = '\n';
 
 	for (s = 1; s < strlen(tail) - 1; s++) {
+		if(tail[s] == '\\') {
+			switch(tail[s + 1]) {
+			case '\"':
+				s++;
+				break;
+			}
+		}
 		to_add[d++] = tail[s];
 	}
 	to_add[d] = 0;
@@ -116,7 +129,7 @@ static void append_line(gchar ** old, const gchar * tail, gboolean continuation)
 	*old = result;
 }
 
-gboolean add_to_obsolete(gchar *comment)
+gboolean add_to_obsolete(GtrPo *po, gchar *comment)
 {
 	if(comment && strstr(comment, "#~ msgid"))
 	{
@@ -146,15 +159,16 @@ gboolean add_to_obsolete(gchar *comment)
 /*
  * The core parsing function for the given po file.
  */ 
-void gtranslator_parse(const gchar *filename)
+GtrPo *gtranslator_parse(const gchar *filename)
 {
+	GtrPo *po;
 	gboolean first_is_fuzzy;
 	gchar *base;
 
-	g_return_if_fail(filename!=NULL);
+	g_return_val_if_fail(filename!=NULL, NULL);
 
 	base=g_path_get_basename(filename);
-	g_return_if_fail(base[0]!='\0');
+	g_return_val_if_fail(base[0]!='\0', NULL);
 	g_free(base);
 
 	po = g_new0(GtrPo, 1);
@@ -178,24 +192,20 @@ void gtranslator_parse(const gchar *filename)
 	 */
 	if(gtranslator_utils_check_file_permissions(po)==FALSE)
 	{
-		return;
+		return NULL;
 	}
 	
-	if (!gtranslator_parse_core())
+	if (!gtranslator_parse_core(po))
 	{
-		gtranslator_po_free();
-		return;
+		gtranslator_po_free(po);
+		return NULL;
 	}
 
 #define FIRST_MSG GTR_MSG(po->messages->data)
 	first_is_fuzzy=(FIRST_MSG->status & GTR_MSG_STATUS_FUZZY) != 0;
 	gtranslator_message_status_set_fuzzy(FIRST_MSG, FALSE);
+	po->fuzzy--;
 	
-	/*
-	 * If the first message is header (it always should be)
-	 */
-	po->header = gtranslator_header_get(FIRST_MSG);
-
 	if (po->header)
 	{
 		GList *header_li;
@@ -211,22 +221,22 @@ void gtranslator_parse(const gchar *filename)
 	else
 	{
 		gtranslator_message_status_set_fuzzy(FIRST_MSG, first_is_fuzzy);
-
-		/*
-		 * Provide a default po->header to avoid segfaults (#62244)
-		 */
-		po->header=gtranslator_header_create_from_prefs();
+		if(first_is_fuzzy)
+			po->fuzzy++;
+		else
+			po->fuzzy--;
 	}
 	
 	file_opened = TRUE;
 	po->file_changed = FALSE;
 	po->length = g_list_length(po->messages);
 
+#ifdef UTF8_CODE
 	/*
 	 * Set the utf8 field of the GtrPo to TRUE if we are editing an UTF-8 
 	 *  encoded file.
 	 */
-	if(gtranslator_utf8_po_file_is_utf8())
+	if(gtranslator_utf8_po_file_is_utf8(po))
 	{
 		po->utf8=TRUE;
 	}
@@ -235,7 +245,7 @@ void gtranslator_parse(const gchar *filename)
 		po->utf8=FALSE;
 	}
 
-	po->locale_charset=gtranslator_utils_get_locale_charset();
+	po->locale_charset=gtranslator_utils_get_locale_charset(po);
 	
 	/* FIXME: converting the messages to UTF8 there since I don't
 	   know the parsing code, and I don't feel like figuring it out
@@ -243,25 +253,31 @@ void gtranslator_parse(const gchar *filename)
 	   properly encoded. Please fix it...
 	*/
 	if (po->utf8 == FALSE) {
-		gtranslator_utf8_convert_po_to_utf8();
+		gtranslator_utf8_convert_po_to_utf8(po);
 	}
+#endif
 
 	/*
 	 * Set the current message to the first message.
 	 */
 	po->current = g_list_first(po->messages);
+
+	return po;
 }
 
-gboolean gtranslator_parse_core(void)
+gboolean gtranslator_parse_core(GtrPo *po)
 {
 	GtrMsg 	*msg;
 	
 	FILE 	*fs;
+	char 	*rawline;
 	char 	*line;
-	
+
 	guint 	 lines=0;
 	guint	 position=-1;
-	
+
+	GError  *errv;
+
 	/*
 	 * If TRUE, means that a corresponding part is read
 	 */
@@ -280,16 +296,48 @@ gboolean gtranslator_parse_core(void)
 	/*
 	 * Parse the file line by line...
 	 */
-	while ((line = gtranslator_utils_getline (fs)) != NULL) {
+	while ((rawline = gtranslator_utils_getline (fs)) != NULL) {
 		
 		lines++;
 
 		/*
-		 * Warn if not valid UTF-8
+		 * Convert lines from charset specified in header
 		 */
-		if (!g_utf8_validate(line, strlen(line), NULL))
-			g_warning("%s:%d: line contains invalid UTF-8", po->filename, lines);
+		if (po->header && po->header->charset) {
+			/*
+			 * Check if the header charset line is in a sane state.
+			 */
+			if(!nautilus_strcmp(po->header->charset, "CHARSET"))
+			{
+				/*
+				 * If the default CHARSET definition is present
+				 *  in the po file header, then we do now simply
+				 *   assume the file to be UTF-8 as it's the
+				 *    default charset for GNOME.
+				 */
+				GTR_FREE(po->header->charset);
+				po->header->charset=g_strdup("UTF-8");
+			}
 
+			/*
+			 * Convert from header charset to UTF-8
+			 */
+			line = g_convert(rawline, strlen(rawline), "utf-8", po->header->charset, NULL, NULL, &errv);
+			if(!line) {
+				g_warning("%s:%d: could not convert line from '%s' to UTF-8: %s", po->filename, lines, po->header->charset, errv->message);
+				continue;
+			}
+		}
+		else {
+			line = rawline;
+		}
+
+       		/*
+		 * Warn if result still not valid UTF-8
+		 */
+		if(!g_utf8_validate(line, strlen(line), NULL))
+			g_warning("%s:%d: line contains invalid UTF-8", po->filename, lines);
+			
 		/*
 		 * If it's a comment, no matter of what kind. It belongs to
 		 * the message pair below
@@ -317,17 +365,14 @@ gboolean gtranslator_parse_core(void)
 				}
 				else if(msg->comment)
 				{
-					if(add_to_obsolete(GTR_COMMENT(msg->comment)->comment))
+					if(add_to_obsolete(po, GTR_COMMENT(msg->comment)->comment))
 					{
 						gtranslator_comment_free(&msg->comment);
 						msg->comment=NULL;
 					}
 				}
-			} else
-			/*
-			 * If it's a msgid
-			 */
-			if (nautilus_str_has_prefix(line, "msgid \"")) {
+			}
+			else if (nautilus_str_has_prefix(line, "msgid \"")) {
 				/*
 				 * This means the comment is completed
 				 */
@@ -335,6 +380,16 @@ gboolean gtranslator_parse_core(void)
 				if (line[8] != '\0')
 					append_line(&msg->msgid, &line[6], FALSE);
 			} 
+			else if (nautilus_str_has_prefix(line, "msgid_plural \""))
+			{
+				/*
+				 * Now we've got a new msgid_plural tag.
+				 */
+				if(line[15]!='\0')
+				{
+					append_line(&msg->msgid_plural, &line[13], FALSE);
+				}
+			}
 			/*
 			 * If it's a msgstr. 
 			 */
@@ -347,6 +402,31 @@ gboolean gtranslator_parse_core(void)
 				if (line[9] != '\0')
 					append_line(&msg->msgstr, &line[7], FALSE);
 			}
+			else if(nautilus_str_has_prefix(line, "msgstr[0] \""))
+			{
+				/*
+				 * Now we've got a msgstr item in here.
+				 */
+				msgid_ok = TRUE;
+				if(line[12]!='\0')
+				{
+					append_line(&msg->msgstr, &line[10], FALSE);
+				}
+			}
+			else if(nautilus_str_has_prefix(line, "msgstr[1] \""))
+			{
+				if(line[12]!='\0')
+				{
+					append_line(&msg->msgstr_1, &line[10], FALSE);
+				}
+			}
+			else if(nautilus_str_has_prefix(line, "msgstr[2] \""))
+			{
+				if(line[12]!='\0')
+				{
+					append_line(&msg->msgstr_2, &line[10], FALSE);
+				}
+			}
 			else
 			/*
 			 * A continuing msgid or msgstr
@@ -357,11 +437,31 @@ gboolean gtranslator_parse_core(void)
 				{
 					append_line(&msg->msgid, line, TRUE);
 				}
+				else if((comment_ok == TRUE) && msgid_ok == FALSE &&
+				    msg->msgid_plural)
+				{
+					append_line(&msg->msgid_plural, line, TRUE);
+				}
 				else if((msgid_ok == TRUE) &&
 					(msgstr_ok == FALSE))
 				{
 					append_line(&msg->msgstr, line, TRUE);
 				}
+				else if((msgid_ok == TRUE) && (msgstr_ok == FALSE))
+				{
+					if(msg->msgstr && !msg->msgstr_1 && !msg->msgstr_2)
+					{
+						append_line(&msg->msgstr, line, TRUE);
+					}
+					else if(msg->msgstr && msg->msgstr_1 && !msg->msgstr_2)
+					{
+						append_line(&msg->msgstr_1, line, TRUE);
+					}
+					else
+					{
+						append_line(&msg->msgstr_2, line, TRUE);
+					}
+				}
 				else if((comment_ok == FALSE) &&
 					(msgid_ok == FALSE) &&
 					(msgstr_ok == FALSE))
@@ -393,9 +493,18 @@ gboolean gtranslator_parse_core(void)
 			
 			position++;
 			msg->no=position;
-			
+
 			po->messages =
 			    g_list_prepend(po->messages, (gpointer) msg);
+
+			/*
+			 * The first message should always be the header. Parse it
+			 * do determine charset to read rest of file in with.
+			 */
+			if(position == 0) {
+				po->header = GTR_HEADER(gtranslator_header_get(FIRST_MSG));
+			}
+
 			/*
 			 * Reset the status of message
 			 */
@@ -417,7 +526,7 @@ gboolean gtranslator_parse_core(void)
 		po->messages = g_list_prepend(po->messages, (gpointer) msg);
 	}
 	else if(msg->comment && msg->comment->comment &&
-		add_to_obsolete(GTR_COMMENT(msg->comment)->comment))
+		add_to_obsolete(po, GTR_COMMENT(msg->comment)->comment))
 	{
 		gtranslator_comment_free(&msg->comment);
 		GTR_FREE(msg);
@@ -442,8 +551,6 @@ gboolean gtranslator_parse_core(void)
  */
 void gtranslator_parse_main(const gchar *filename)
 {
-  gchar 	*title;
-  gchar         *base;
 	/*
 	 * Test if such a file does exist.
 	 */
@@ -460,18 +567,17 @@ void gtranslator_parse_main(const gchar *filename)
 	 */
 	if(gtranslator_utils_check_file_being_open(filename))
 	{
-		return;
-	}
-	else
-	{
-		gtranslator_config_set_string("runtime/filename", 
-			(gchar *) filename);
+		if(!open_anyway)
+			return;
 	}
 
+	gtranslator_config_set_string("runtime/filename", 
+				      (gchar *) filename);
+
 	/*
 	 * Use the new core function.
 	 */
-	gtranslator_parse(filename);
+	po = gtranslator_parse(filename);
 
 	/*
 	 * Iterate to the main GUI thread -- well, no locks for the GUI should
@@ -531,10 +637,16 @@ void gtranslator_parse_main(const gchar *filename)
 		gtranslator_actions_enable(ACT_REMOVE_ALL_TRANSLATIONS);
 	}
 
-	gtranslator_application_bar_update(0);
-	
 	if(po->header)
 	{
+		/*
+		 * Fix b.g.o #120596 via checking for our table to be present.
+		 */
+		if(GtrPreferences.show_messages_table && !po->table_already_created)
+		{
+			gtranslator_messages_table_create();
+		}
+
 		/*
 		 * If we've got still some default values,
 		 * substitute with values from preferences,
@@ -560,8 +672,28 @@ void gtranslator_parse_main(const gchar *filename)
 		/* But PACKAGE and VERSION should be entered by user */
 		gtranslator_header_edit_dialog(NULL, NULL);
 	}
+}
+
+/*
+ * GUI-related stuff. Should only be called when the GUI needs updating. Could
+ * probably also be moved to 'gui.c'.
+ */
+void gtranslator_parse_main_extra()
+{	
+	gchar         *base;
+	gchar 	*title;
 	
-	gtranslator_message_show(po->current->data);
+	g_return_if_fail(po!=NULL);
+
+	if(!po->header) {
+		/*
+		 * Provide a default po->header to avoid segfaults (#62244)
+		 */
+		g_warning(_("Header record not found - defaulting from preferences"));
+		po->header=gtranslator_header_create_from_prefs();
+	}
+  
+	gtranslator_application_bar_update(0);
 
 	/*
 	 * Test if the filename is NOT equivalent to our temp files' names.
@@ -589,9 +721,16 @@ void gtranslator_parse_main(const gchar *filename)
 	 */
 	if(GtrPreferences.show_messages_table)
 	{
+		gtranslator_messages_table_clear();
 		gtranslator_messages_table_create();
 	}
 
+	/*
+	 * Show the current message and avoid a crash with editing plurals
+	 *  without having any active TreeModel present.
+	 */
+	gtranslator_message_show(po->current->data);
+
 	/*
 	 * Hook up the autosaving function if wished.
 	 */
@@ -601,7 +740,6 @@ void gtranslator_parse_main(const gchar *filename)
 			(GtrPreferences.autosave_timeout * 60000),
 			(GSourceFunc) gtranslator_utils_autosave, NULL);
 	}
-	
 }
 
 void gtranslator_parse_the_file_from_file_dialog(GtkWidget * widget, gpointer of_dlg)
@@ -640,10 +778,15 @@ static gchar *restore_msg(const gchar * given)
 
 	rest = g_string_sized_new(strlen(given));
 	for (s = 0; s < strlen(given); s++) {
-		if (given[s] == '\n') {
+		switch(given[s]) {
+		case '\"':
+			g_string_append(rest, "\\\"");
+			break;
+		case '\n':
 			g_string_append(rest, "\"\n\"");
 			lines++;
-		} else {
+			break;
+		default:
 			g_string_append_c(rest, given[s]);
 		}
 	}
@@ -661,6 +804,8 @@ static void write_the_message(gpointer data, gpointer fs)
 	GString *string;
 	gchar 	*id;
 	gchar	*str;
+	gchar   *line;
+	GError  *errv;
 
 	msg=GTR_MSG(data);
 
@@ -693,18 +838,73 @@ static void write_the_message(gpointer data, gpointer fs)
 	GTR_FREE(id);
 
 	/*
-	 * Preface for the msgstr -- the content comes below.
+	 * Check if we've got a pluram forms message.
 	 */
-	string=g_string_append(string, "\"\nmsgstr \"");
+	if(po->header->plural_forms && msg->msgid_plural && msg->msgstr)
+	{
+		string=g_string_append(string, "\"\nmsgid_plural \"");
+
+		id=restore_msg(msg->msgid_plural);
+		string=g_string_append(string, id);
+		GTR_FREE(id);
+
+		id=restore_msg(msg->msgstr);
+
+		string=g_string_append(string, "\"\nmsgstr[0] \"");
+		string=g_string_append(string, id);
+		GTR_FREE(id);
+
+		if(msg->msgstr_1)
+		{
+			id=restore_msg(msg->msgstr_1);
+
+			string=g_string_append(string, "\"\nmsgstr[1] \"");
+			string=g_string_append(string, id);
+			GTR_FREE(id);
+		}
+
+		if(msg->msgstr_2)
+		{
+			id=restore_msg(msg->msgstr_2);
+
+			string=g_string_append(string, "\"\nmsgstr[2] \"");
+			string=g_string_append(string, id);
+			GTR_FREE(id);
+		}
+	}
+	else
+	{
+		/*
+		 * Preface for the msgstr -- the content comes below.
+		 */
+		string=g_string_append(string, "\"\nmsgstr \"");
 	
-	str=restore_msg(msg->msgstr);
-	string=g_string_append(string, str);
-	GTR_FREE(str);
+		str=restore_msg(msg->msgstr);
+		string=g_string_append(string, str);
+		GTR_FREE(str);
+	}
+	
+	/*
+	 * Convert lines from charset specified in header
+	 */
+	if (po->header && po->header->charset) {
+		/*
+		 * Convert from header charset to UTF-8
+		 */
+		line = g_convert(string->str, strlen(string->str), po->header->charset, "utf-8", NULL, NULL, &errv);
+		if(!line) {
+			g_warning("could not convert message number '%d' from UTF-8 to '%s' to UTF-8: %s", msg->pos, po->header->charset, errv->message);
+			line = string->str;
+		}
+	}
+	else {
+		line = string->str;
+	}
 	
 	/*
 	 * Write the string content and the newlines to our write stream.
 	 */
-	fprintf((FILE *) fs, "%s\"\n\n", string->str);
+	fprintf((FILE *) fs, "%s\"\n\n", line);
 	
 	g_string_free(string, TRUE);
 }
@@ -824,13 +1024,17 @@ Your file should likely be named '%s.po'."),
 	/* If user GtrPreferences to, warn it about fuzzy mesages left */
 	if(GtrPreferences.warn_if_fuzzy && po->fuzzy)
 	{
-		gchar *warn;
-		warn = g_strdup_printf(_("File %s\n"
-				       "contains %d fuzzy messages"),
-				       po->filename, po->fuzzy);
-		gnome_warning_dialog_parented(warn, 
-			GTK_WINDOW(gtranslator_application));
-		GTR_FREE(warn);
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new(
+			GTK_WINDOW(gtranslator_application),
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_MESSAGE_WARNING,
+			GTK_BUTTONS_OK,
+			_("File %s\ncontains %d fuzzy messages"),
+			po->filename, po->fuzzy);
+		gtk_dialog_run(GTK_DIALOG(dialog));
+		gtk_widget_destroy(dialog);
 	}
 
 	return TRUE;
@@ -865,10 +1069,10 @@ void gtranslator_save_current_file_dialog(GtkWidget * widget, gpointer useless)
 /*
  * Frees the po variable
  */
-void gtranslator_po_free(void)
+void gtranslator_po_free(GtrPo *po)
 {
-	if(!po)
-		return;
+	g_return_if_fail(po!=NULL);
+
 	if (po->messages) 
 	{
 		g_list_foreach(po->messages, gtranslator_message_free, NULL);
@@ -893,10 +1097,10 @@ void gtranslator_file_close(GtkWidget * widget, gpointer useless)
 	/*
 	 * If user doesn't know what to do with changed file, return
 	 */
-	if (!gtranslator_should_the_file_be_saved_dialog)
+	if (!gtranslator_should_the_file_be_saved_dialog())
 		return;
 	
-	gtranslator_po_free();
+	gtranslator_po_free(po);
 	gtranslator_undo_clean_register();
 	
 	file_opened = FALSE;
@@ -910,6 +1114,7 @@ void gtranslator_file_close(GtkWidget * widget, gpointer useless)
 		gtranslator_messages_table_clear();
 	}
 
+#ifdef NOT_PORTED
 	/*
 	 * Stop gtkspell.
 	 */ 
@@ -917,7 +1122,8 @@ void gtranslator_file_close(GtkWidget * widget, gpointer useless)
 	{
 		gtkspell_stop();
 	}
-	
+#endif /* NOT_PORTED */	
+
 	gtranslator_text_boxes_clean();
 	gtk_label_set_text(GTK_LABEL(extra_content_view->comment), "");
 	gtk_widget_set_sensitive(GTK_WIDGET(extra_content_view->edit_button), FALSE);
@@ -950,23 +1156,9 @@ void gtranslator_file_revert(GtkWidget * widget, gpointer useless)
 {
 	gchar *save_this;
 	if (po->file_changed) {
-		GtkWidget *dialog;
-		gchar *question;
-		gint reply;
-		question =
-		    g_strdup_printf(
-				    _("File %s\nwas changed. Do you want to revert to saved copy?"),
-				    po->filename);
-		dialog =
-		    gnome_message_box_new(question,
-					  GNOME_MESSAGE_BOX_QUESTION,
-					  GNOME_STOCK_BUTTON_YES,
-					  GNOME_STOCK_BUTTON_NO,
-					  GNOME_STOCK_BUTTON_CANCEL, NULL);
-		gtranslator_dialog_show(&dialog, "gtranslator -- revert");
-		reply = gnome_dialog_run(GNOME_DIALOG(dialog));
-		GTR_FREE(question);
-		if (reply != GNOME_YES)
+		guint reply;
+		reply = gtranslator_file_revert_dialog(NULL, po->filename);
+		if (reply != GTK_RESPONSE_YES)
 			return;
 	}
 	save_this = g_strdup(po->filename);
@@ -976,6 +1168,7 @@ void gtranslator_file_revert(GtkWidget * widget, gpointer useless)
 	po->file_changed = FALSE;
 	gtranslator_file_close(NULL, NULL);
 	gtranslator_parse_main(save_this);
+	gtranslator_parse_main_extra();
 	GTR_FREE(save_this);
 }
 
@@ -1011,6 +1204,7 @@ static void remove_translation(gpointer message, gpointer useless)
 	if(msg->status & GTR_MSG_STATUS_FUZZY)
 	{
 		gtranslator_message_status_set_fuzzy(msg, FALSE);
+		po->fuzzy--;
 		internal_change=TRUE;
 	}
 
