@@ -27,8 +27,7 @@
 #include <config.h>
 #endif
 
-#include "defines.include"
-#include "dialogs.h"
+#include "defines.h"
 #include "gui.h"
 #include "parse.h"
 #include "prefs.h"
@@ -38,7 +37,7 @@
 
 #include <gtk/gtkmain.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <libgnomeui/gnome-app-util.h>
 
 /*
  * The update function.
@@ -51,31 +50,14 @@ void update(GtkWidget *widget, gpointer useless)
 	gint res=200;
 	gchar *command;
 	gchar *newfile;
-
-	if(file_opened && po->file_changed)
-	{
-		GtkWidget 	*dialog;
-		gint		reply;
-		
-		dialog=gnome_message_box_new(_("The po file was changed and the changes weren't saved yet. Do you still want to update the po file now?"),
-		GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
-		GNOME_STOCK_BUTTON_CANCEL, NULL);
-		gtranslator_dialog_show(&dialog, "gtranslator -- save the current file before updating?");
-
-		reply=gnome_dialog_run(GNOME_DIALOG(dialog));
-		if(reply!=GNOME_YES)
-		{
-			return;
-		}
-	}
 	
 	/*
 	 * Build this magical line.
 	 */
 	command=g_strdup_printf("'%s' '%s' '%s' '%s' 2>&1 1>/dev/null",
 		SCRIPTSDIR "/my-update.sh",
-		g_basename(po->filename),
-		g_dirname(po->filename),
+		g_path_get_basename(po->filename),
+		g_path_get_dirname(po->filename),
 		po->header->prj_name);
 	
 	/*
@@ -148,6 +130,7 @@ void update(GtkWidget *widget, gpointer useless)
 	 * Parse the file again.
 	 */
 	gtranslator_parse_main(newfile);
+	gtranslator_parse_main_extra();
 	GTR_FREE(command);
 	GTR_FREE(newfile);
 }
