@@ -61,7 +61,7 @@ gchar *gtranslator_utils_get_raw_file_name(const gchar *filename)
 	g_return_val_if_fail(filename!=NULL, g_strdup(""));
 
 	o=g_string_new("");
-	filename=g_basename(filename);
+	filename=g_path_get_basename(filename);
 
 	/*
 	 * Strip all extensions after the _first_ point.
@@ -217,7 +217,7 @@ gchar *gtranslator_utils_get_language_name_by_locale_code(const gchar *locale_co
 		{
 			return _(languages[i].name);
 		}
-		else if(!g_strncasecmp(languages[i].locale, locale_code, 2))
+		else if(!g_ascii_strncasecmp(languages[i].locale, locale_code, 2))
 		{
 			return _(languages[i].name);
 		}
@@ -603,7 +603,7 @@ gfloat gtranslator_utils_calculate_similarity(const gchar *a, const gchar *b)
 	 */
 	if(strlen(a) > strlen(b))
 	{
-		if(!g_strncasecmp(a, b, (strlen(a) - 1)))
+		if(!g_ascii_strncasecmp(a, b, (strlen(a) - 1)))
 		{
 			return 100.0;
 		}
@@ -612,7 +612,7 @@ gfloat gtranslator_utils_calculate_similarity(const gchar *a, const gchar *b)
 	}
 	else
 	{
-		if(!g_strncasecmp(b, a, (strlen(b) - 1)))
+		if(!g_ascii_strncasecmp(b, a, (strlen(b) - 1)))
 		{
 			return 100.0;
 		}
@@ -628,6 +628,7 @@ gfloat gtranslator_utils_calculate_similarity(const gchar *a, const gchar *b)
 	{
 		if((a[i] <= 32) || (b[i] <= 32))
 		{
+			i++;
 			continue;
 		}
 		else if(a[i]==b[i])
@@ -690,7 +691,7 @@ gint gtranslator_utils_stringlist_strcasecmp(GList *list, const gchar *string)
 /*
  * Get the right localename for the language.
  */
-gchar *gtranslator_utils_get_locale_name(void)
+gchar *gtranslator_utils_get_locale_name(GtrPo *po)
 {
 	if((!file_opened) || !(po->header->language))
 	{
@@ -719,10 +720,11 @@ gchar *gtranslator_utils_get_locale_name(void)
 	}
 }
 
+#ifdef UTF8_CODE
 /*
  * Get the right charset/encoding for the language.
  */
-gchar *gtranslator_utils_get_locale_charset(void)
+gchar *gtranslator_utils_get_locale_charset(GtrPo *po)
 {
 	if((!file_opened) || !(po->header->language))
 	{
@@ -792,6 +794,7 @@ gchar *gtranslator_utils_get_locale_charset(void)
 		return g_strdup(_("iso-8859-1"));
 	}
 }
+#endif
 
 /*
  * Set up the lists to use within the combo boxes.
@@ -870,37 +873,6 @@ gboolean gtranslator_utils_language_lists_free(GtkWidget  * widget, gpointer use
 	return FALSE;
 }
 
-/*
- * Try to convert the < 0.37 color preferences to their new location.
- */
-void gtranslator_utils_old_colors_to_new_location()
-{
-	gchar *value;
-	gboolean converted=FALSE;
-	
-	value=gtranslator_config_get_string("colors/fg");
-
-	if(value && value[0]=='#')
-	{
-		gtranslator_config_set_string("colors/own_fg", value);
-		converted=TRUE;
-	}
-
-	value=gtranslator_config_get_string("colors/bg");
-
-	if(value && value[0]=='#')
-	{
-		gtranslator_config_set_string("colors/own_bg", value);
-		converted=TRUE;
-	}
-
-	if(converted)
-	{
-		gtranslator_config_set_string("colors/fg", "black");
-		gtranslator_config_set_string("colors/bg", "white");
-	}
-}
-
 /* gtranslator_utils_getline
  *
  * reads newline (or CR) or EOF terminated line from stream, allocating the
@@ -987,8 +959,8 @@ gchar *gtranslator_utils_get_full_language_name(gchar *lang)
 			 * The first two characters are converted to small
 			 *  characters, and the last two are capitalized.
 			 */
-			language=g_string_down(language);
-			taillanguage=g_string_up(taillanguage);
+			language=g_string_ascii_down(language);
+			taillanguage=g_string_ascii_up(taillanguage);
 
 			/*
 			 * Append the capilized letters after a '_'.
@@ -1062,7 +1034,7 @@ gchar *gtranslator_xml_get_string_prop_by_name_with_default(const xmlNode *paren
 	}
 	else
 	{
-		g_return_if_fail(default_string!=NULL);
+		g_return_val_if_fail(default_string!=NULL, NULL);
 		ret_val=g_strdup (default_string);
 	}
 
