@@ -20,6 +20,7 @@
 #include "dialogs.h"
 #include "gui.h"
 #include "nautilus-string.h"
+#include "parse.h"
 #include "prefs.h"
 #include "runtime-config.h"
 #include "utils.h"
@@ -76,7 +77,7 @@ gchar *gtranslator_utils_invert_dot(gchar *str)
 			gunichar c = g_utf8_get_char(p);
 			if (c == middot)
 				g_string_append_c(newstr, ' ');
-			else if (g_unichar_isspace(c))
+			else if (g_unichar_break_type(c) == G_UNICODE_BREAK_SPACE)
 				g_string_append_unichar(newstr, middot);
 			else
 				g_string_append_unichar(newstr, c);
@@ -114,7 +115,8 @@ void gtranslator_utils_save_geometry(void)
  */
 void gtranslator_utils_restore_geometry(gchar  * gstr)
 {
-	gint x, y, width, height;
+	gint x=0, y=0, width=0, height=0;
+
 	/*
 	 * Set the main window's geometry from the preferences.
 	 */
@@ -144,9 +146,9 @@ void gtranslator_utils_restore_geometry(gchar  * gstr)
 		}
 	}
 	if (x != -1)
-		gtk_widget_set_uposition(gtranslator_application, x, y);
+		gtk_window_move(GTK_WINDOW(gtranslator_application), x, y);
 	if ((width > 0) && (height > 0))
-		gtk_window_set_default_size(GTK_WINDOW(gtranslator_application), width, height);
+		gtk_window_resize(GTK_WINDOW(gtranslator_application), width, height);
 }
 
 GtkWidget *gtranslator_utils_attach_combo_with_label(GtkWidget  * table, gint row,
@@ -164,7 +166,7 @@ GtkWidget *gtranslator_utils_attach_combo_with_label(GtkWidget  * table, gint ro
 	if (value)
 		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), value);
 	
-	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry), editable);
+	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(combo)->entry), editable);
 	
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row + 1);
 	gtk_table_attach_defaults(GTK_TABLE(table), combo, 1, 2, row, row + 1);
@@ -235,17 +237,17 @@ GtkWidget *gtranslator_utils_attach_text_with_label(GtkWidget  * table, gint row
 		gtk_text_buffer_get_start_iter(buff, &start);
 		gtk_text_buffer_insert(buff, &start, value, strlen(value));
 		gtk_text_view_set_buffer(GTK_TEXT_VIEW(widget), buff);
+		if(callback)
+		{
+			g_signal_connect(G_OBJECT(buff), "changed",
+					 G_CALLBACK(callback), NULL);
+		}
+
 	}
 	gtk_container_add(GTK_CONTAINER(scroll), widget);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row + 1);
 	gtk_table_attach_defaults(GTK_TABLE(table), scroll, 1, 2, row, row + 1);
 
-	if(callback)
-	{
-		g_signal_connect(G_OBJECT(widget), "changed",
-			         G_CALLBACK(callback), NULL);
-	}
-
 	return widget;
 }
 
@@ -338,17 +340,6 @@ GtkWidget *gtranslator_utils_attach_color_with_label(GtkWidget *table,
 	return color_selector;
 }
 
-GtkWidget *gtranslator_utils_append_page_to_preferences_dialog(GtkWidget  * probox, gint rows, gint cols,
-			     const char *label_text)
-{
-	GtkWidget *label;
-	GtkWidget *page;
-	label = gtk_label_new(label_text);
-	page = gtk_table_new(rows, cols, FALSE);
-	gnome_property_box_append_page(GNOME_PROPERTY_BOX(probox), page, label);
-	return page;
-}
-
 /*
  * Checks the given file for read permissions first and then
  *  for the right write permissions.
@@ -405,7 +396,6 @@ permission for it."),
 	return TRUE;
 }
 
-void gtranslator_parse_main(const gchar *filename);
 void open_it_anyway_callback( gint, gpointer );
 
 void open_it_anyway_callback( gint reply, gpointer filename )
@@ -415,7 +405,6 @@ void open_it_anyway_callback( gint reply, gpointer filename )
 	switch( reply ){
 		case 0: /* YES */
 			open_anyway = TRUE;
-			gtranslator_parse_main( (gchar*)filename );
 			break;
 		case 1: /* NO */
 			break;
@@ -424,7 +413,7 @@ void open_it_anyway_callback( gint reply, gpointer filename )
 }
 
 /*
- * Check if the given file is alrady opened by gtranslator.
+ * Check if the given file is already opened by gtranslator.
  */
 gboolean gtranslator_utils_check_file_being_open(const gchar *filename)
 {
@@ -448,34 +437,10 @@ gboolean gtranslator_utils_check_file_being_open(const gchar *filename)
 	if(resultfilename && (!nautilus_strcasecmp(resultfilename, filename)) &&
 		(strlen(resultfilename)==strlen(filename)))
 	{
-		gchar *error_message;
-		gchar *fname;
-		
-		fname = g_strdup( filename );
-
-		error_message=g_strdup_printf(
-		_("The file\n"
-		"\n"
-		"   %s\n"
-		"\n"
-		"is already open in another instance of gtranslator!\n"
-		"Please close the other instance of gtranslator handling\n"
-		"this file currently to re-gain access to this file.\n"
-		"\n"
-		"Shall fool gtranslator open this file anyway ?"), 
-		filename);
-
-		/*
-		 * Return with FALSE if the given file is already open. ???
-		 * +++++
-		 */
-		/* gnome_app_error(GNOME_APP(gtranslator_application), error_message); */
-		gnome_app_question_modal( GNOME_APP(gtranslator_application), error_message,
-			open_it_anyway_callback, (gpointer)fname );
-		
-		GTR_FREE(resultfilename);
-		GTR_FREE(error_message);
-
+		gint reply;
+		reply = gtranslator_already_open_dialog(NULL, (gpointer)filename);
+		if(reply == GTK_RESPONSE_YES)
+			open_anyway = TRUE;
 		return TRUE;
 	}
 	else
